/*
	MakePass 4.0 - Stand Alone password reply generator

  	To be used in Packet Radio to make reply for user access authentication
  	process

  	This program was written and maintained during the years 1992~1998
  	by Marco Savegnago IW3FQG

  	This is the application object implementation
*/

#include "gpmsg.h"
#include "gpapps.h"
#include "gppass.h"

const char TMakePass::thisVersion[] = "4.0";

TMakePass::TMakePass( int na, char **a, ostream&	ostream )
:
	out(ostream),
   psz_callsign(NULL),
   psz_passwordFile(NULL),
   psz_userCmd(NULL),
   b_stripSSID(false)
{
	bool	b_printBanner	=	true;

	if ( na < 4)
   {
   	banner();
      usage();
   }

   //	Save the path

	fnsplit( a[ 0 ], sz_currentDrive, sz_currentDir, sz_currentName, NULL );

	//	Check options and parameter

   for ( int narg = 1; narg < na; narg++ )
   {
      char* p=a[narg];

      if ( *p == '-' || *p == '/')
      {
          p++;

          switch(toupper(*p))
          {
              case '?':
              case 'H':
              		banner();
                  usage();

              case 'N':	// don't print banner
              		b_printBanner=false;
                  break;

              case 'S': // strip SSID
                  b_stripSSID=true;
                  break;

              default: // unknow switch
                  out << "\aError! Invalid switch <" << --p << ">. Type /? for Help." << endl;
                  exit(1);
                  break;
          }
      }
      else
      {
      	if(psz_callsign == NULL)
         {
         	psz_callsign=a[narg];
         }
			else if(psz_passwordFile == NULL)
         {
         	psz_passwordFile=a[narg];
         }
   		else if(psz_userCmd == NULL)
         {
         	psz_userCmd=a[narg];
         }
      }
   }

   if(b_printBanner)
   	banner();
}


void TMakePass::banner()
{
   out << endl
       << "MakePass 4.0 - Stand Alone password reply generator"
       << endl
       << "Written by Marco Savegnago IW3FQG " << __DATE__  << endl
       << endl;
}

void TMakePass::usage()
{
	char usage_str[] =

     "Make Packet Radio BBS Password Reply for User and BBS Authentication\n"
     "This is am HAMWARE program, so it can be copied/modify freely but not for\n"
     "commercial purposes!\n\n"

     "Usage MakePass [/|-options] <callsign> <passwordfile> <challenge>\n"
     "options:\n"
     "/N - suppress banner\n"
     "/S - strip away SSID from callsign before processing.\n\n"
     "Typical usage: makepass I3KUH-8 MAKEPASS.PWD \"? 0 1 2 3 4 5 [012345789]\"\n"
     ;

	out << usage_str << endl;

   exit(2);
}



bool TMakePass::Run()
{
   if(b_stripSSID)
	{
		register int i;

		for( i = 0; psz_callsign[ i ] && psz_callsign[ i ] != '-'; i++ );

		psz_callsign[ i ] = EOS;
	}

	Password password ( psz_callsign, psz_passwordFile);

   switch(password.ReadPassword())
   {
      case	0:
      {
      	switch(password.MakeReply(psz_userCmd, out))
         {
         	case	0:
            	return true;

      		case	1:
      			out	<<	"\aChallange " << psz_userCmd << " NOT found!" << endl;
         		break;

         	default:
      			out	<<	"\aI/O error. Exit." << endl;
      			break;
         }
         break;
      }
      case	1:
      {
      	out	<<	"\aPassword for " << psz_callsign << " NOT found!" << endl;
         break;
      }
      default:
      {
      	out	<<	"\aI/O error. Exit." << endl;
      	break;
      }
   }

   return false;
}

